/* jornal - a dead simple command like diary */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

/* BEGIN CUSTOMIZATION BLOCK */

/* The path/name relative to $HOME where your .jornal file will be saved. Defaults to ".jornal" */
/* ex: set to ".config/diary" to set it to $HOME/.config/diary */
const char *jornal_file_path_from_home = ".jornal";

/* END CUSTOMIZATION BLOCK */

#define DATE_BUFFER_SIZE 12

int main(int argc, char *argv[]) {
	int i, j, ch;
	int newline = 1;
	time_t epoch;
	struct tm now, recent;
	char *jornal_file_path;
	char *home_path;
	char date_buffer[DATE_BUFFER_SIZE];
	int date_count = 0;
	FILE *jornal_file;

	epoch = time(NULL);
	now = *localtime(&epoch);

	home_path = getenv("HOME");

	if (home_path == NULL) {
		fprintf(stderr, "jornal: Could not get env variable 'HOME'");
		exit(1);
	}

	/* Create the .jornal file path by concatenating $HOME/ and the path to the jornal file (default to .jornal) */
	jornal_file_path = malloc(sizeof(char) * (strlen(home_path) + strlen(jornal_file_path_from_home) + 2));

	for (i = 0; home_path[i] != '\0'; ++i) {
		jornal_file_path[i] = home_path[i];
	}

	jornal_file_path[i] = '/';
	++i;

	for (j = 0; jornal_file_path_from_home[j] != '\0'; ++j) {
		jornal_file_path[i + j] = jornal_file_path_from_home[j];
		jornal_file_path[i + j + 1] = '\0';
	}

	jornal_file = fopen(jornal_file_path, "r");

	if (jornal_file == NULL) {
		/* TODO: Just make the file if it doesn't exist instead of crashing */
		fprintf(stderr, "jornal: Could not open file '%s'\n", jornal_file_path);
		exit(1);
	}

	/* Get most recent date from jornal_file */
	for (i = 0; i < DATE_BUFFER_SIZE; ++i) {
		date_buffer[i] = '\0';
	}

	while ((ch = getc(jornal_file)) != EOF) {
		if (newline) {
			newline = 0;

			if (ch == '~') {
				j = 0;
				for (;;) {
					ch = getc(jornal_file);
					if (ch == EOF || ch == '\n' || j >= DATE_BUFFER_SIZE - 1) {
						break;
					}

					date_buffer[j] = ch;
					++j;
				}

				date_buffer[j + 1] = '\0';
				++date_count;
			}
		}

		if (ch == '\n') {
			newline = 1;
		}
	}

	fclose(jornal_file);

	/* Process most recent date */
	if (date_count == 0) {
		/* TODO: Better handle when there's no date. Actually test it, I should say */
		recent.tm_year = now.tm_year;
		recent.tm_mon = now.tm_mon;
		recent.tm_mday = now.tm_mday;
	} else {
		int i, j;
		char field_buffer[6];
		int matched_delim = 0;
		int date_field = 0; /* 0 == YYYY; 1 == MM; 2 == DD */

		for (i = 0, j = 0; date_buffer[i] != '\0'; ++i) {
			if (date_buffer[i] == '-') {
				if (date_field == 0) {
					recent.tm_year = atoi(field_buffer) - 1900;
				}

				if (date_field == 1) {
					recent.tm_mon = atoi(field_buffer) - 1;
				}

				++date_field;
				matched_delim = 1;
				j = 0;
			}

			if (!matched_delim) {
				field_buffer[j] = date_buffer[i];
				field_buffer[j + 1] = '\0';
				++j;
			} else {
				matched_delim = 0;
			}
		}

		/* Dirty hack to get the last field */
		recent.tm_mday = atoi(field_buffer);
	}

	jornal_file = fopen(jornal_file_path, "a");

	/* Check to see if we're still on the same date as today. */
	if (recent.tm_mday != now.tm_mday || recent.tm_mon != now.tm_mon || recent.tm_year != now.tm_year) {
		fprintf(jornal_file, "\n~%d-%02d-%02d\n", now.tm_year + 1900, now.tm_mon + 1, now.tm_mday);
	}

	/* Write entry to file */
	fprintf(jornal_file, "%02d:%02d:%02d ", now.tm_hour, now.tm_min, now.tm_sec);

	if (argc > 1) {
		for (i = 1; i < argc; ++i) {
			fprintf(jornal_file, "%s ", argv[i]);
		}
	} else {
		/* If there's no args, than take the first line of standard input */
		int ch;

		for (;;) {
			ch = getchar();

			if (ch == EOF || ch == '\n') {
				break;
			}

			putc(ch, jornal_file);
		}
	}
	fprintf(jornal_file, "\n");

	fclose(jornal_file);

	free(jornal_file_path);

	return 0;
}
